{+-----------------------------------------------------------------------------+}
{| Author: Philip Bairstow.                                                    |}
{| Date: 09-10-00.                                                             |}
{| Version: 3.0                                                                |}
{| E-mail: Philip@pbairstow.com                                                |}
{| Url: www.pbairstow.com                                                      |}
{| ICQ: 47850165                                                               |}
{| Yahoo: Tekno1999                                                            |}
{|                                                                             |}
{| Comment: I would like to thank the following people for there bug reports   |}
{|          and encouragement to create this new version with bug fixes:       |}
{|          DouWe, Bill Tel, Greg, Irman Fauzi and Stephen Nickson.            |}
{|                                                                             |}
{|          This version fixes bugs in functions KeyExists and ValueExists     |}
{|          as well as some improvements to the compoent and Error Trapping.   |}
{|          Also to aid simplicity there are now 2 components not 1 that       |}
{|          Does Every thing.                                                  |}
{|                                                                             |}
{|                                                                             |}
{|                                                                             |}
{+-----------------------------------------------------------------------------+}

unit EasyRegAdvanced;

interface

uses
  Windows, Messages, SysUtils,
  Classes, Controls, Graphics,
  Forms, Dialogs, Registry;

type
  TRootKey = (bkHKEY_LOCAL_MACHINE, bkHKEY_CLASSES_ROOT, bkHKEY_CURRENT_USER,
    bkHKEY_USERS, bkHKEY_CURRENT_CONFIG);

  TEasyRegException = class(ERegistryException);

  TEasyRegAdvanced = class(TComponent)
  private
    FReg: TRegistry;
  protected
    FRootKey: HKEY;
    FOnCreate: TNotifyEvent;
    FOnActivate: TNotifyEvent;
    FOnDestroy: TNotifyEvent;
    FRegKey: TRootKey;
    FRegString: String;
    FRegBaseKey: TRootKey;
    procedure WMCreate(var Message: TWMCreate); message WM_CREATE;
    procedure WMActivate(var Message: TWMActivate); message WM_ACTIVATE;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    function Crypt(s: string; EncryptKey:Integer): string;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure WriteString(Name, Value: String);
    function ReadString(Name: String): String;
    procedure WriteEncryptedString(Name, Value: String; Code: Integer);
    function ReadEncryptedString(Name: String; Code: Integer): String;
    procedure SetBaseKey(Key: TRootKey);
    procedure WriteInteger(Name: String; Value: Integer);
    function ReadInteger(Name: String): Integer;
    procedure WriteBoolean(Name: String; Value: Boolean);
    function ReadBoolean(Name: String): Boolean;
    function KeyExists(Name: String): Boolean;
    function ValueExists(Name: String): Boolean;
    function DeleteKey(Name: String): Boolean;
    function DeleteValue(Name: String): Boolean;
    procedure WriteCurrency(Name: String; Value: Currency);
    function ReadCurrency(Name: String): Currency;
    procedure WriteFloat(Name: String; Value: Double);
    function ReadFloat(Name: String): Double;
    procedure RunOnce(ApplicationPath: String);
  published
    property OnCreate:       TNotifyEvent   read FOnCreate      write FOnCreate;
    property OnActivate:     TNotifyEvent   read FOnActivate    write FOnActivate;
    property OnDestroy:      TNotifyEvent   read FOnDestroy     write FOnDestroy;
    property RegString:      String         read FRegString     write FRegString;
    property RegBaseKey:     TRootKey       read FRegBaseKey    write FRegBaseKey;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('PBSoftware', [TEasyRegAdvanced]);
end;

constructor TEasyRegAdvanced.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TEasyRegAdvanced.Destroy;
begin
  inherited;
end;

{+-----------------------------------------------------------------------------+}
{| Procedures triggered by Windows Messages(WM)                                |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WMCreate;
begin
  if Assigned(FOnCreate) then FOnCreate(Self);
end;

procedure TEasyRegAdvanced.WMActivate;
begin
  if Assigned(FOnActivate) then FOnActivate(Self);
end;

procedure TEasyRegAdvanced.WMDestroy;
begin
  if Assigned(FOnDestroy) then FOnDestroy(Self);
end;

{+-----------------------------------------------------------------------------+}
{| Run Once.                                                                   |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.RunOnce(ApplicationPath: String);
begin
  FReg := TRegistry.Create;
    try
      FReg.RootKey := HKEY_CURRENT_USER;
      if FReg.OpenKey('\Software\Microsoft\Windows\CurrentVersion\RunOnce', True)
        then FReg.WriteString(ApplicationPath,'"' + ParamStr(0) + '"');
    finally
      FReg.CloseKey;
      FReg.Free;
    end;
end;
 
{+-----------------------------------------------------------------------------+}
{| Private procedures and functions used by TEasyReg.                          |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.SetBaseKey(Key: TRootKey);
begin
  FRootKey := HKEY_CLASSES_ROOT;
  Case Key of

    bkHKEY_CLASSES_ROOT:
        FRootKey := HKEY_CLASSES_ROOT;
    bkHKEY_CURRENT_USER:
        FRootKey := HKEY_CURRENT_USER;
    bkHKEY_LOCAL_MACHINE:
        FRootKey := HKEY_LOCAL_MACHINE;
    bkHKEY_USERS:
        FRootKey := HKEY_USERS;
    bkHKEY_CURRENT_CONFIG:
        FRootKey := HKEY_CURRENT_CONFIG;
   end;
end;

function TEasyRegAdvanced.Crypt(s: string; EncryptKey:Integer): string;
var
  tmpstr: string;
  x: integer;
  b: byte;
begin
  tmpstr:= '';
  for x:= 1 to length(S) do
  begin
    b:= not (byte(S[x]) xor EncryptKey);
    tmpstr:= tmpstr + char(b);
  end;
  result:= tmpstr;
end;

{+-----------------------------------------------------------------------------+}
{| Public procedures and functions.                                            |}
{| Procedure for Writing and function for Reading String Value.                |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteString(Name, Value: String);
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.RootKey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            FReg.WriteString(Name, Value);
            if not FReg.ValueExists(Name) then
              raise TEasyRegException.Create('Error: Value not Created');
        finally
          FReg.CloseKey;
          FReg.Free;
         end

      except
        On E:ERegistryException do Raise;
     end;
   end;
end;

function TEasyRegAdvanced.ReadString(Name: String): String;
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            Result := FReg.ReadString(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
       end;

      except
        On E:ERegistryException do Raise;
    end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Procedure for Writing and Function for Reading a Encrypted String Value.    |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteEncryptedString(Name, Value: String; Code: Integer);
var
  tmp: String;
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      tmp := Crypt(Value,Code);
      try
        FReg := TRegistry.Create;
        try
          FReg.RootKey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            FReg.WriteString(Name, tmp);
            if not FReg.ValueExists(Name) then
              raise TEasyRegException.Create('Error: Value not Created');
        finally
          FReg.CloseKey;
          FReg.Free;
         end

      except
        On E:ERegistryException do Raise;
     end;
   end;
end;

function TEasyRegAdvanced.ReadEncryptedString(Name: String; Code: Integer): String;
var
  tmp: String;
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            tmp := FReg.ReadString(Name);
            Result := Crypt(tmp,Code);

        finally
          FReg.CloseKey;
          FReg.Free;
       end;

      except
        On E:ERegistryException do Raise;
    end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Procedures for Writing and Function for Reading a Integer Value.            |}
{------------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteInteger(Name: String; Value: Integer);
begin
   if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
      try
        FReg.Rootkey := FRootKey;
        if FReg.OpenKey(Trim(FRegString), True) then
          FReg.WriteInteger(Name, Value);
          if not FReg.ValueExists(Name) then
            raise TEasyRegException.Create('Error: Value not Created');

      finally
        FReg.CloseKey;
        FReg.Free;
       end;
    except
      On E:ERegistryException do Raise;
    end;
  end;
end;

function TEasyRegAdvanced.ReadInteger(Name: String): Integer;
begin
   Result := -1;
   if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            Result := FReg.ReadInteger(Name);

      finally
        FReg.CloseKey;
        FReg.Free;
      end;

   except
     On E:ERegistryException do Raise;
    end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Procedure for Reading and Function for Writing a Boolean Value.             |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteBoolean(Name: String; Value: Boolean);
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            FReg.WriteBool(Name, Value);
            if not FReg.ValueExists(Name) then
              raise TEasyRegException.Create('Error: Value not Created');
        finally
          FReg.CloseKey;
          FReg.Free;
        end;

      except
        On E:ERegistryException do Raise;
      end;
   end;
end;

function TEasyRegAdvanced.ReadBoolean(Name: String): Boolean;
begin
  Result := False;
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
             Result := FReg.ReadBool(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Procedure for Reading and Function for Writing a Currency Value.            |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteCurrency(Name: String; Value: Currency);
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            FReg.WriteCurrency(Name, Value);
            if not FReg.ValueExists(Name) then
              raise TEasyRegException.Create('Error: Value not Created');
        finally
          FReg.CloseKey;
          FReg.Free;
        end;

      except
        On E:ERegistryException do Raise;
      end;
   end;
end;

function TEasyRegAdvanced.ReadCurrency(Name: String): Currency;
begin
  Result := $-1;
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
             Result := FReg.ReadCurrency(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Procedure for Reading and Function for Writing a Real Value.                |}
{+-----------------------------------------------------------------------------+}

procedure TEasyRegAdvanced.WriteFloat(Name: String; Value: Double);
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            FReg.WriteFloat(Name, Value);
            if not FReg.ValueExists(Name) then
              raise TEasyRegException.Create('Error: Value not Created');
        finally
          FReg.CloseKey;
          FReg.Free;
        end;

      except
        On E:ERegistryException do Raise;
      end;
   end;
end;

function TEasyRegAdvanced.ReadFloat(Name: String): Double;
begin
  Result := -1;
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
             Result := FReg.ReadFloat(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Function for Testing if a Key Exists.                                       |}
{+-----------------------------------------------------------------------------+}

function TEasyRegAdvanced.KeyExists(Name: String): Boolean;
begin
  Result := False;
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if FReg.OpenKey(Trim(FRegString), True) then
            if FReg.KeyExists(Name) then
               Result := True;

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Function for Testing if Value Exists.                                       |}
{+-----------------------------------------------------------------------------+}

function TEasyRegAdvanced.ValueExists(Name: String): Boolean;
begin
  Result := False;
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          if (FReg.OpenKey(Trim(FRegString), True)) and (Name <> '') then
             Result := FReg.ValueExists(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

{+-----------------------------------------------------------------------------+}
{| Now time for some procedures and functions to Clean Up.                     |}
{+-----------------------------------------------------------------------------+}

function TEasyRegAdvanced.DeleteKey(Name: String): Boolean;
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          Result := FReg.DeleteKey(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;

function TEasyRegAdvanced.DeleteValue(Name: String): Boolean;
begin
  if not Length(FRegString) > 0 then
    raise TEasyRegException.Create('Error: No RegString Provided')
      else
    begin
      SetBaseKey(FRegBaseKey);
      try
        FReg := TRegistry.Create;
        try
          FReg.Rootkey := FRootKey;
          Result := FReg.DeleteKey(Name);

        finally
          FReg.CloseKey;
          FReg.Free;
        end;

    except
      On E:ERegistryException do Raise;
     end;
  end;
end;


end.
